* MakeView.Prg		(c) 2001 Peisch Custom Software, Inc.

*  This program demonstrates an easier way to create that DBC in code, along with the views.

*  I always keep a separate DBC for my views.
*  The advantage to this is that when I change the structure of a table, I don't have to worry about
*  finding all the views that use that table, and fix them.  I can just erase the DBC and let the program
*  rebuild the views when it starts.  This code is designed to run when the DBC is missing.

*  There are some rules you must follow:
*     1. The primary key of the table that you can update via the view must be the first
*        field of the view.
*     2. You must populate an array called aFlds with the structure of the updateable fields from your view, with
*        the PK in the first row.  There are 3 techniques for doing this.  Each one is illustrated in the examples below.
*     3. Create your DBC and build your code according to the examples below.  Then, call the SetView Properties
*        function, sending it at least the first 2 parameters.  (See the top of the SetViewFunction for a
*        description of the parameters.)

* EXAMPLES
* You must substitute the code shown here with code to generate the DBC and views you need for your program.
if not file('MyNewViews.DBC')
	CLOSE DATA ALL
	CREATE DATABASE 'MYNEWVIEWS.DBC'

	***************** View setup for CUSTORDS ***************
	* This is an example of a simple, non-parameterized view
	CREATE SQL VIEW "CUSTORDS" ; 
		AS SELECT Customers.* ;
		FROM  Customers ;
		ORDER BY Customers.custname
	* Populate aFlds with the information about the updateable fields from the view
	dimension aFlds[1]		&& so we don't get an error that aFlds isn't an array
	* Because we're just using all the fields from the Customers table, we can open that table
	* to get the structure.  This would also work if the viewed used all the fields from one table, plus
	* additional fields from a second table, but only the fields in the first table are updateable.
	select Customers
	afields('aFlds')		&& Populate the array with the field definitions
	use in Customers

	SetViewProperties('CustOrds','OrdHead',.T.)

	***************** View setup for CUSTORDS2 ***************
	* This is an example of a simple, parameterized view
	CREATE SQL VIEW "CUSTORDS2" ; 
		AS SELECT Ordhead.ordheadid, Ordhead.orderno, Ordhead.orddate, Customers.custname ;
		FROM customers ;
			INNER JOIN ordhead ;
				ON Customers.custid = Ordhead.custid ;
		WHERE Ordhead.orddate >= ?lOrderDate ;
		ORDER BY Customers.custname
	* Populate aFlds with the information about the updateable fields from the view
	* Since we're only updating some fields from a table, it doesn't make sense to populate aFlds by using the
	* source table.  Since only 1 field from this view isn't updateable, if we make that the last field, we can
	* use the view to populate the aFlds array, then remove the last row.
	dimension aFlds[1]		&& Clear out anything that may have been there from before
	use CustOrds2 nodata
	afields('aFlds')		&& Populate the array with the field definitions
	use in CustOrds2
	* Now remove the last row from the array, since that field isn't updateable.
	* Note: afields will create a 4 column array for free tables, but a 16 column array for items in a DBC
	dimension aFlds[alen(aFlds,1)-1,16]

	SetViewProperties('CustOrds2','OrdHead',.T.)

	***************** View setup for CUSTORDS3 ***************
	* This is an example of a parmeterized view which uses multiple tables
	CREATE SQL VIEW "CUSTORDS3" ; 
		AS SELECT OrdItems.OrdItemId, OrdItems.QtyOrd, Ordhead.orderno, Ordhead.orddate, Ordhead.ordheadid, ;
				Customers.custname, Inventory.Descrip ;
		FROM Customers ;
			INNER JOIN ordhead ;
				ON Customers.custid = Ordhead.custid ;
			INNER JOIN orditems ;
				ON Ordhead.ordheadid = Orditems.ordheadid ;
			INNER JOIN inventory ;
				ON Orditems.invenid = Inventory.invenid ;
		WHERE Ordhead.orddate >= ?lOrderDate ;
		ORDER BY Customers.custname
	* Since there are only 2 fields from this view that we want to be updateable, it's easier just to populate
	* the array manually.
	dimension aFlds[2,4]
	aFlds[1,1] = 'OrditemId'
	aFlds[1,2] = 'I'
	aFlds[1,3] = 4
	aFlds[1,4] = 0
	aFlds[2,1] = 'QtyOrd'
	aFlds[2,2] = 'N'
	aFlds[2,3] = 3
	aFlds[2,4] = 0

	SetViewProperties('CustOrds3','OrdItems',.T.)
endif

ENDFUNC
*-------------------------------------------------------------------------------------
FUNCTION SetViewProperties
lparameters pViewName, pSourceTable, pPKUpdateable
* Parameters are:
*    pViewName = The name of the view we're setting properties for
*    pSourceTable = The name of the source table of the view that is updateable via the view
*    pPKUpdateable = .T. if the primary key of the source table is updateable

local li

if empty(pViewName) or empty(pSourceTable)
	messagebox('You must send the view and source table names to the SetViewProperties function!',48,'Invalid parameters')
	return
endif
if type('pViewName') <> 'C' or type('pSourceTable') <> 'C'
	messagebox('pViewName and pSourceTable parameters for SetViewProperties must be character strings!',48,'Invalid parameters')
	return
endif
if type('aFlds[1]') <> 'C'
	messagebox('You must populate the aFlds array with the structure of the fields to be updated!',48,'Missing array')
	return
endif
if type('pPKUpdateable') <> 'L'
	messagebox('The third parameter must be a logical!',48,'Invalid parameters')
	return
endif

DBSetProp(pViewName, 'View', 'UpdateType', 1)
DBSetProp(pViewName, 'View', 'WhereType', 3)
DBSetProp(pViewName, 'View', 'FetchMemo', .T.)
DBSetProp(pViewName, 'View', 'SendUpdates', .T.)
DBSetProp(pViewName, 'View', 'UseMemoSize', 255)
DBSetProp(pViewName, 'View', 'FetchSize', 100)
DBSetProp(pViewName, 'View', 'MaxRecords', -1)
DBSetProp(pViewName, 'View', 'Tables', pSourceTable)
DBSetProp(pViewName, 'View', 'Prepared', .F.)
DBSetProp(pViewName, 'View', 'CompareMemo', .T.)
DBSetProp(pViewName, 'View', 'FetchAsNeeded', .F.)
DBSetProp(pViewName, 'View', 'FetchSize', 100)
DBSetProp(pViewName, 'View', 'Comment', "")
DBSetProp(pViewName, 'View', 'BatchUpdateCount', 1)
DBSetProp(pViewName, 'View', 'ShareConnection', .F.)

for li = 1 to alen(aFlds,1)
	if li = 1
		dbsetprop(pViewName+'.'+aFlds[li,1], 'Field', 'KeyField', .T.)
	else
		dbsetprop(pViewName+'.'+aFlds[li,1], 'Field', 'KeyField', .F.)
	endif
	if li <> 1 or (pPKUpdateable)
		dbsetprop(pViewName+'.'+aFlds[li,1], 'Field', 'Updatable', .T.)
		dbsetprop(pViewName+'.'+aFlds[li,1], 'Field', 'UpdateName', pSourceTable+'.'+aFlds[li,1])
		dbsetprop(pViewName+'.'+aFlds[li,1], 'Field', 'DataType', ;
				aFlds[li,2]+'('+ltrim(str(aFlds[li,3]))+','+ltrim(str(aFlds[li,4]))+')')
	else
		dbsetprop(pViewName+'.'+aFlds[li,1], 'Field', 'Updatable', .F.)
	endif
endfor

endfunc
*----------------------------------------------------------------------------
